% -------------------------------------------------------------------------
% Blade element program
% Even Hjetland, spring 2009
% -------------------------------------------------------------------------

clear all
clc

% uiimport foilxy.hog

hogprof=[1,0;0.9789,-0.0018;0.9536,-0.0019;0.9268,-0.0021;0.8984,-0.0026;0.8685,-0.0033;0.8373,-0.0042;0.8047,-0.0052;0.771,-0.0064;0.7363,-0.0077;0.7008,-0.0092;0.6647,-0.0107;0.6282,-0.0123;0.5915,-0.0139;0.5549,-0.0155;0.5186,-0.0172;0.4827,-0.0187;0.4475,-0.0203;0.4132,-0.0217;0.38,-0.0231;0.3479,-0.0244;0.3171,-0.0257;0.2877,-0.0268;0.2597,-0.0279;0.2333,-0.0289;0.2083,-0.0298;0.1849,-0.0306;0.163,-0.0314;0.1426,-0.032;0.1237,-0.0325;0.1061,-0.0328;0.0899,-0.033;0.0749,-0.0329;0.0612,-0.0326;0.0486,-0.0319;0.037,-0.0306;0.0264,-0.0281;0.0168,-0.0242;0.008,-0.018;0,0;0.008,0.0309;0.0168,0.0431;0.0264,0.053;0.037,0.062;0.0486,0.0704;0.0612,0.0784;0.0749,0.086;0.0899,0.0932;0.1061,0.1;0.1237,0.1063;0.1426,0.1121;0.163,0.1171;0.1849,0.1213;0.2083,0.1246;0.2333,0.1266;0.2597,0.1268;0.2877,0.1243;0.3171,0.1187;0.3479,0.1111;0.38,0.1028;0.4132,0.0942;0.4475,0.0855;0.4827,0.0768;0.5186,0.0682;0.5549,0.06;0.5915,0.0521;0.6282,0.0446;0.6647,0.0377;0.7008,0.0312;0.7363,0.0253;0.771,0.0201;0.8047,0.0154;0.8373,0.0114;0.8685,0.0081;0.8984,0.0053;0.9268,0.0033;0.9536,0.0018;0.9789,0.0013;1,0;];
hogprof_orig=hogprof;


% Declaring dimentions

Di=0.4;                         % Diameter hub in meter
Do=1.10;                        % Diameter fan casing in meter
l=1.13;                         % Length intake
rps=1450/60;                    % Rounds per second
Q=25;                           % Volum flow in cubic meter per second
ct=3e-3;                        % tip clearance between impeller tip and casing

rho=1.225;                      % Density of air in kg per cubic meter
mu=1.81e-5;                     % Kinematic viscosity

Z=5;                            % Number of impeller blades
n=20;                           % number of elements
r=Di/2:(Do-Di)/(2*n):Do/2;      % Radius of each element
dr=(Do-Di)/(2*n);
t=2*pi*r/Z;                     % Ratio of distance between blades and chord length
A_el=t*dr;                      % Area of an element normal W_inf direction
A_imp=Z*sum(A_el);              % Projected area of impeller in axial direction
Cl_i=1.09;                      % Value of Cl at hub
Cl_o=1.06;                      % Value of Cl at blade tip
Cl=Cl_i:(Cl_o-Cl_i)/n:Cl_o;     % Vector of Cl values
aoa=8;                          % Angle of attack
Cd=0.03;                        % Drag coefficient from impeller blades
dp_imp_p=10;    % Estimated viscous pressure loss over impeller as % of dynamic pressure


% Calculating resulting parameters

nu=Di/Do;                               % Ratio between diameter of hub and fan casing
A_in=pi*(Do/2)^2;                       % Area of inlet
omega=rps*2*pi;                         % Angular velocity                            
U_in=Q/A_in;                            % Speed of flow at inlet

Re_i=U_in*rho*Do/(mu*2);                % Reynoldsnumber in in- and outlet
delta_f=1.83*l/sqrt(Re_i);              % Displacement thickness
A_in_new=pi*(Do/2-delta_f)^2;           % New effective area
U_in_f=(A_in/A_in_new)*U_in;            % New effective axial velocity 
C1=(A_in/A_imp)*U_in_f;                 % New effective axial velocity at impeller
dp_io=2*0.0195*l*0.5*rho*(C1^2)/Do;     % Viscous pressure loss in- and outlet

p_dyn=0.5*rho*C1^2;                     % Dynamic pressure for flow through impeller
dp_imp=p_dyn*dp_imp_p/100;
dp_r=p_dyn+dp_imp+dp_io;                % Estimated required pressure difference produced over impeller

Uo=omega*Do/2;                          % Radial velocity at blade tip
psi=2*dp_r/(rho*Uo^2);                  % Pressure number (temporary value)
phi=4*Q/(pi*Uo*Do^2);                   % Delivery number (temporary value)
sigma=phi^0.5/(psi^0.75); %#ok<NASGU>   % Specific rotational speed (temporary value)
delta=psi^0.25/(phi^0.5); %#ok<NASGU>   % Specific diameter (temporary value)

P_r=dp_r*Q;                             % Required power to provide the desired axial velocity
const=4*pi*dp_r/(rho*omega*Z);          % Chord length ditribution constant



% Deciding the property distribution per element

U_rot=omega*r;                      % Rotational velocity of the impeller
dC=(dp_r)./(rho*U_rot);             % Rotational velocity generated by the impeller
W_inf=sqrt((U_rot-dC/2).^2+C1^2);   % Velocity felt by the blade
beta_inf=atand(C1./(U_rot-dC/2));   % Geometric twist
c=const./(W_inf.*Cl);


% Calculating average element values from nodal values

c_d=zeros(1,n);
area=zeros(1,n);
for i=2:n+1
    c_d(i-1)=(c(i-1)+c(i))/2; %#ok<AGROW>
    area(i-1)=dr*c_d(i-1); %#ok<AGROW>
    W_inf_d(i-1)=(W_inf(i-1)+W_inf(i))/2; %#ok<AGROW>
    beta_inf_d(i-1)=(beta_inf(i-1)+beta_inf(i))/2; %#ok<AGROW>
end


% Calculating the over all performances of the impeller


for i=1:n    
    dp_imp_i(i)=0.5*rho*W_inf_d(i)^2*Cd*area(i)*W_inf_d(i)/Q; %#ok<AGROW>
end

dp_imp_new=Z*sum(dp_imp_i);         % New value of viscous loss over impeller
dp_r=p_dyn+dp_imp_new+dp_io;        % New value of reqired pressure produced over impeller


% Iterating to solve the viscous pressure loss over impeller

j=2;
dp_imp_v(1)=dp_imp;
dp_imp_v(2)=dp_imp_new;

while abs(dp_imp_v(j)-dp_imp_v(j-1))>=1e-20

    Uo=omega*Do/2;                      % Radial velocity at blade tip
    P_r=dp_r*Q;                         % Required power to provide the desired axial velocity
    const=4*pi*dp_r/(rho*omega*Z);      % Chord length ditribution constant

    % Deciding the property distribution per element

    dC=dp_r./(rho*U_rot);               % Rotational velocity generated by the impeller
    W_inf=sqrt((U_rot-dC/2).^2+C1^2);   % Velocity felt by the blade
    beta_inf=atand(C1./(U_rot-dC/2));   % Geometric twist
    c=const./(W_inf.*Cl);               % Chord length
    Re=rho*W_inf.*c./mu;                % Reynolds number
    ang=beta_inf+aoa;                   % Total geometric twist
    t=2*pi*r/Z;                         % Distance between blades
    t_c=t./c;                           % Ratio of distance between blades and chord length
    A_el=t*dr;                          % Area of one element in axial direction
    
    % Calculating average values for elements from nodal values

    for i=2:n+1
        c_d(i-1)=(c(i-1)+c(i))/2; %#ok<AGROW>
        area(i-1)=dr*c_d(i-1); %#ok<AGROW>  % Area of an element on blade surface
        W_inf_d(i-1)=(W_inf(i-1)+W_inf(i))/2; %#ok<AGROW>
        beta_inf_d(i-1)=(beta_inf(i-1)+beta_inf(i))/2; %#ok<AGROW>
        U_rot_d(i-1)=(U_rot(i-1)+U_rot(i-1))/2; %#ok<AGROW>
        dC_d(i-1)=(dC(i-1)+dC(i))/2; %#ok<AGROW>
        A_el_d(i-1)=(A_el(i-1)+A_el(i))/2; %#ok<AGROW>
        r_d(i-1)=(r(i-1)+r(i))/2; %#ok<AGROW>
        Re_d(i-1)=(Re(i-1)+Re(i))/2; %#ok<AGROW>
    end
    A_imp=Z*sum(A_el_d);
    beta_1=atand(C1./U_rot_d);
    beta_2=atand(C1./(U_rot_d-dC_d));
    W1=sqrt((U_rot_d.^2+C1^2));
    W2=sqrt((U_rot_d-dC_d).^2+C1^2);
    C2=sqrt(C1^2+dC_d.^2);

    % Calculating the over all performances of the impeller, elementwise


    for i=1:n
        dFy_i(i)=0.5*rho*W_inf_d(i)^2*(Cl(i)*cosd(beta_inf_d(i))-Cd*sind(beta_inf_d(i)))*area(i); %#ok<AGROW>
        dp_imp_i(i)=0.5*rho*W_inf_d(i)^2*Cd*area(i)*W_inf_d(i)/Q; %#ok<AGROW>
        dFx_i(i)=0.5*rho*W_inf_d(i)^2*(Cl(i)*sind(beta_inf_d(i))+Cd*cosd(beta_inf_d(i)))*area(i); %#ok<AGROW>
        dKy_i(i)=rho*0.5*((W1(i)*cosd(beta_1(i)))^2-(W2(i)*cosd(beta_2(i)))^2)*A_el_d(i); %#ok<AGROW>
        dKx_i(i)=rho*C1*dC_d(i)*A_el_d(i); %#ok<AGROW>
        dp_tot_i(i)=rho*dC_d(i)*U_rot_d(i)*A_el_d(i); %#ok<AGROW>
        dp_dyn_i(i)=0.5*rho*(C2(i)^2-C1^2)*A_el_d(i); %#ok<AGROW>
    end
    
    dp_imp_new=Z*sum(dp_imp_i); % New value of viscous pressure loss over impeller
   
    j=j+1;
    dp_imp_v(j)=dp_imp_new; %#ok<AGROW>
    dp_r=p_dyn+dp_imp_new+dp_io; %#ok<AGROW> 
    residual(j)=dp_imp_v(j)-dp_imp_v(j-1); %#ok<AGROW>
    
end



% Stating final performance results

dp_imp=dp_imp_new;      % Viscous pressure loss over impeller   
Fy=Z*sum(dFy_i);        % Force on impeller in axial direction
Fx=Z*sum(dFx_i);        % Force on impeller in tangential direction
Ky=Z*sum(dKy_i);        % Sum of force on blade elements in axial direction
Kx=Z*sum(dKx_i);        % Sum of force on blade elements in tangential direction
T=Z*dFx_i*r_d';         % Torque acting on the blade about the y-axis


dp_dyn=5*sum(dp_dyn_i)/A_imp;       % Dynamic pressure difference over impeller
dp_tot=5*sum(dp_tot_i)/A_imp;       % Total pressure difference over impeller
dp_stat=Ky/A_imp;                   % Static pressure difference over impeller


P_i=T*omega;                        % Power delivered to impeller by motor
P_fl=abs(dp_tot)*Q;                 % Power delivered to stream by impeller
F_s=Fy/P_i;                         % Spesific aksial force

eta_imp=(P_fl/P_i);                 % Impeller efficiency       
eta_h=1-(dp_imp/dp_tot);            % Hydraulic efficiency    

psi=2*abs(dp_stat)/(rho*Uo^2);      % Pressure number
sigma=phi^0.5/(psi^0.75);           % Specific rotational speed
delta=psi^0.25/(phi^0.5);           % Specific diameter
eta_tot_imp=0.80;                   % Total efficiency of impeller (accounting 
                                    % for effects not implemented in this program

        
Cq=Q/(rps*Do^3);                % Dimensionless constant
Ch=dp_stat/(rho*rps^2*Do^2);    % Dimensionless constant
Cp=P_i/(rho*rps^3*Do^5);       % Dimensionless constant


disp(['Skyvkraft i aksiell retning fra impeller p strmning:   ' num2str(Ky) ' N'])
disp(['Kraft i tverretning:                                     ' num2str(abs(Fx)) ' Pa'])
disp(['Moment om aksial retning:                                ' num2str(abs(T)) ' Pa'])
disp(['Statisk trykkkning over impeller:                       ' num2str(abs(dp_stat)) ' Pa'])
disp(['Dynamisk trykkkning over impeller:                      ' num2str(abs(dp_dyn)) ' Pa'])
disp(['Total trykkkning over impeller:                         ' num2str(abs(dp_tot)) ' Pa'])
disp(['Effekt tilfrt impeller fra motor:                       ' num2str(P_i) ' W'])
disp(['Effekt tilfrt strmningsmediet fra impeller:            ' num2str(P_fl) ' W'])
disp(['Virkningsgrad hos impeller:                              ' num2str(eta_imp)])
disp(['Spesifik skyvkraft:                                      ' num2str(F_s)])
disp(['Hydraulisk virkningsgrad:                                ' num2str(eta_h)])

% Generating the profile files

profmat=zeros(length(hogprof(:,1)),3*n);
hogprof(:,1)=hogprof(:,1)-0.25;

for i=1:n+1
    profmat(:,3*i-2)=c(i)*hogprof(:,1);
    profmat(:,3*i-1)=c(i)*hogprof(:,2);
    profmat(:,3*i)=r(i);
end

profmat_temp=profmat;

for i=1:n+1
    profmat_temp(:,3*i-2)=profmat(:,3*i-2)*cosd(ang(i))+profmat(:,3*i-1)*sind(ang(i));
    profmat_temp(:,3*i-1)=-profmat(:,3*i-2)*sind(ang(i))+profmat(:,3*i-1)*cosd(ang(i));
    profmat(:,3*i-2)=profmat_temp(:,3*i-2);
    profmat(:,3*i-1)=profmat_temp(:,3*i-1);
end

prof1=[profmat(:,1),profmat(:,2),profmat(:,3)];
prof2=[profmat(:,4),profmat(:,5),profmat(:,6)];
prof3=[profmat(:,7),profmat(:,8),profmat(:,9)];
prof4=[profmat(:,10),profmat(:,11),profmat(:,12)];
prof5=[profmat(:,13),profmat(:,14),profmat(:,15)];
prof6=[profmat(:,16),profmat(:,17),profmat(:,18)];
prof7=[profmat(:,19),profmat(:,20),profmat(:,21)];
prof8=[profmat(:,22),profmat(:,23),profmat(:,24)];
prof9=[profmat(:,25),profmat(:,26),profmat(:,27)];
prof10=[profmat(:,28),profmat(:,29),profmat(:,30)];
prof11=[profmat(:,31),profmat(:,32),profmat(:,33)];
prof12=[profmat(:,34),profmat(:,35),profmat(:,36)];
prof13=[profmat(:,37),profmat(:,38),profmat(:,39)];
prof14=[profmat(:,40),profmat(:,41),profmat(:,42)];
prof15=[profmat(:,43),profmat(:,44),profmat(:,45)];
prof16=[profmat(:,46),profmat(:,47),profmat(:,48)];
prof17=[profmat(:,49),profmat(:,50),profmat(:,51)];
prof18=[profmat(:,52),profmat(:,53),profmat(:,54)];
prof19=[profmat(:,55),profmat(:,56),profmat(:,57)];
prof20=[profmat(:,58),profmat(:,59),profmat(:,60)];
prof21=[profmat(:,61),profmat(:,62),profmat(:,63)];

K=zeros(2,length(prof1(:,1)),3);
K(1,:,1)=prof1(:,1);
K(1,:,2)=prof1(:,2);
K(1,:,3)=prof1(:,3);
K(2,:,1)=prof2(:,1);
K(2,:,2)=prof2(:,2);
K(2,:,3)=prof2(:,3);
K(3,:,1)=prof3(:,1);
K(3,:,2)=prof3(:,2);
K(3,:,3)=prof3(:,3);
K(4,:,1)=prof4(:,1);
K(4,:,2)=prof4(:,2);
K(4,:,3)=prof4(:,3);
K(5,:,1)=prof5(:,1);
K(5,:,2)=prof5(:,2);
K(5,:,3)=prof5(:,3);
K(6,:,1)=prof6(:,1);
K(6,:,2)=prof6(:,2);
K(6,:,3)=prof6(:,3);
K(7,:,1)=prof7(:,1);
K(7,:,2)=prof7(:,2);
K(7,:,3)=prof7(:,3);
K(8,:,1)=prof8(:,1);
K(8,:,2)=prof8(:,2);
K(8,:,3)=prof8(:,3);
K(9,:,1)=prof9(:,1);
K(9,:,2)=prof9(:,2);
K(9,:,3)=prof9(:,3);
K(10,:,1)=prof10(:,1);
K(10,:,2)=prof10(:,2);
K(10,:,3)=prof10(:,3);
K(11,:,1)=prof11(:,1);
K(11,:,2)=prof11(:,2);
K(11,:,3)=prof11(:,3);
K(12,:,1)=prof12(:,1);
K(12,:,2)=prof12(:,2);
K(12,:,3)=prof12(:,3);
K(13,:,1)=prof13(:,1);
K(13,:,2)=prof13(:,2);
K(13,:,3)=prof13(:,3);
K(14,:,1)=prof14(:,1);
K(14,:,2)=prof14(:,2);
K(14,:,3)=prof14(:,3);
K(15,:,1)=prof15(:,1);
K(15,:,2)=prof15(:,2);
K(15,:,3)=prof15(:,3);
K(16,:,1)=prof16(:,1);
K(16,:,2)=prof16(:,2);
K(16,:,3)=prof16(:,3);
K(17,:,1)=prof17(:,1);
K(17,:,2)=prof17(:,2);
K(17,:,3)=prof17(:,3);
K(18,:,1)=prof18(:,1);
K(18,:,2)=prof18(:,2);
K(18,:,3)=prof18(:,3);
K(19,:,1)=prof19(:,1);
K(19,:,2)=prof19(:,2);
K(19,:,3)=prof19(:,3);
K(20,:,1)=prof20(:,1);
K(20,:,2)=prof20(:,2);
K(20,:,3)=prof20(:,3);
K(21,:,1)=prof21(:,1);
K(21,:,2)=prof21(:,2);
K(21,:,3)=prof21(:,3);

% meshing the blade

cla
hold on
axis equal
grid on
surfl(K(:,:,1),K(:,:,2),K(:,:,3),[-1,-0.5,1])
hold off

% saving profile files

save prof1.dat prof1 -ASCII
save prof2.dat prof2 -ASCII
save prof3.dat prof3 -ASCII
save prof4.dat prof4 -ASCII
save prof5.dat prof5 -ASCII
save prof6.dat prof6 -ASCII
save prof7.dat prof7 -ASCII
save prof8.dat prof8 -ASCII
save prof9.dat prof9 -ASCII
save prof10.dat prof10 -ASCII
save prof11.dat prof11 -ASCII
save prof12.dat prof12 -ASCII
save prof13.dat prof13 -ASCII
save prof14.dat prof14 -ASCII
save prof15.dat prof15 -ASCII
save prof16.dat prof16 -ASCII
save prof17.dat prof17 -ASCII
save prof18.dat prof18 -ASCII
save prof19.dat prof19 -ASCII
save prof20.dat prof20 -ASCII
save prof21.dat prof21 -ASCII
